@@ -56,6 +56,7 @@ extern int node_timeout;
 extern int ping_timeout;
 extern int seed_fb_interval;
 extern int sync_timeout;
+extern int clusterer_enable_rerouting;
 
 int dispatch_jobs = 1;
 
@@ -1095,6 +1096,10 @@ void bin_rcv_cl_extra_packets(bin_packet_t *packet, int packet_type,
 		lock_release(node->lock);
 
 	if (dest_id != current_id) {
+		if (clusterer_enable_rerouting == 0) {
+			LM_WARN("Received message for destination id [%d] but rerouting disabled\n", dest_id);
+			goto exit;
+		}
 		/* route the message */
 		bin_push_int(packet, cluster_id);
 		bin_push_int(packet, source_id);
@@ -1877,4 +1882,4 @@ unsigned long clusterer_get_num_nodes(int state)
 	lock_stop_read(cl_list_lock);
 
 	return nodecount;
-}
+}
\ No newline at end of file
