@@ -51,7 +51,7 @@
 
 /* module functions */
 static int mod_init();
-static int destroy(void);
+static void destroy(void);
 
 static mi_response_t *mi_list_root_path(const mi_params_t *params,
 						struct mi_handler *async_hdl);
@@ -59,6 +59,7 @@ static mi_response_t *mi_list_root_path(const mi_params_t *params,
 int port = 8888;
 str ip = {NULL, 0};
 str buffer = {NULL, 0};
+unsigned int hd_conn_timeout_s = 30;
 str tls_cert_file = {NULL, 0};
 str tls_key_file = {NULL, 0};
 str tls_ciphers = {"SECURE256:+SECURE192:-VERS-ALL:+VERS-TLS1.2", 45};
@@ -78,6 +79,7 @@ static const param_export_t params[] = {
 	{"port",          INT_PARAM, &port},
 	{"ip",            STR_PARAM, &ip.s},
 	{"buf_size",      INT_PARAM, &buffer.len},
+	{"conn_timeout",  INT_PARAM, &hd_conn_timeout_s},
 	{"post_buf_size", INT_PARAM, &post_buf_size},
 	{"tls_cert_file", STR_PARAM, &tls_cert_file.s},
 	{"tls_key_file", STR_PARAM,  &tls_key_file.s},
@@ -187,7 +189,7 @@ static int mod_init(void)
 }
 
 
-int destroy(void)
+static void destroy(void)
 {
 	struct httpd_cb *cb = httpd_cb_list;
 
@@ -198,7 +200,6 @@ int destroy(void)
 		shm_free(cb);
 		cb = httpd_cb_list;
 	}
-	return 0;
 }
 
 
