@@ -107,8 +107,8 @@ static struct script_route_ref *trace_filter_route_ref = NULL;
 static int mod_init(void);
 static int proto_wss_init(struct proto_info *pi);
 static int proto_wss_init_listener(struct socket_info *si);
-static int proto_wss_send(struct socket_info* send_sock,
-		char* buf, unsigned int len, union sockaddr_union* to,
+static int proto_wss_send(const struct socket_info* send_sock,
+		char* buf, unsigned int len, const union sockaddr_union* to,
 		unsigned int id);
 static int wss_read_req(struct tcp_connection* con, int* bytes_read);
 static int wss_conn_init(struct tcp_connection* c);
@@ -202,14 +202,14 @@ static int proto_wss_init(struct proto_info *pi)
 	pi->tran.dst_attr		= tcp_conn_fcntl;
 
 	pi->net.flags			= PROTO_NET_USE_TCP;
-	pi->net.read			= (proto_net_read_f)wss_read_req;
+	pi->net.stream.read		= wss_read_req;
 
-	pi->net.conn_init		= wss_conn_init;
-	pi->net.conn_clean		= ws_conn_clean;
+	pi->net.stream.conn.init	= wss_conn_init;
+	pi->net.stream.conn.clean	= ws_conn_clean;
 	if (cert_check_on_conn_reusage)
-		pi->net.conn_match		= tls_conn_extra_match;
+		pi->net.stream.conn.match	= tls_conn_extra_match;
 	else
-		pi->net.conn_match		= NULL;
+		pi->net.stream.conn.match	= NULL;
 	pi->net.report			= wss_report;
 
 	return 0;
@@ -303,8 +303,6 @@ static int wss_conn_init(struct tcp_connection* c)
 	if (!dom) {
 		LM_ERR("no TLS %s domain found\n",
 				(c->flags&F_CONN_ACCEPTED?"server":"client"));
-		c->proto_data = NULL;
-		shm_free(d);
 		return -1;
 	}
 
@@ -390,8 +388,8 @@ static void wss_report(int type, unsigned long long conn_id, int conn_flags,
 
 
 /*! \brief Finds a tcpconn & sends on it */
-static int proto_wss_send(struct socket_info* send_sock,
-		char* buf, unsigned int len, union sockaddr_union* to,
+static int proto_wss_send(const struct socket_info* send_sock,
+		char* buf, unsigned int len, const union sockaddr_union* to,
 		unsigned int id)
 {
 	struct tcp_connection *c;
@@ -497,8 +495,8 @@ send_it:
 
 	/* mark the ID of the used connection (tracing purposes) */
 	last_outgoing_tcp_id = c->id;
-	send_sock->last_local_real_port = c->rcv.dst_port;
-	send_sock->last_remote_real_port = c->rcv.src_port;
+	send_sock->last_real_ports->local = c->rcv.dst_port;
+	send_sock->last_real_ports->remote = c->rcv.src_port;
 
 	tcp_conn_release(c, 0);
 	return n;
