@@ -52,6 +52,18 @@ int src_init(void)
 	}
 #endif
 
+	if (srec_b2b.register_cb(src_event_received,
+			B2BCB_RECV_EVENT, &mod_name) < 0) {
+		LM_ERR("could not register SIPREC event receive callback!\n");
+		return -1;
+	}
+
+	if (srec_b2b.register_cb(src_event_trigger,
+			B2BCB_TRIGGER_EVENT, &mod_name) < 0) {
+		LM_ERR("could not register SIPREC event trigger callback!\n");
+		return -1;
+	}
+
 	skip_failover_codes.len = strlen(skip_failover_codes.s);
 	if (!skip_failover_codes.len)
 		return 0;
@@ -201,6 +213,12 @@ int srec_register_callbacks(struct src_sess *sess)
 			srec_dlg_write_callback, sess, NULL))
 		LM_WARN("cannot register callback for session serialization! "
 			"Will not be able to end siprec session in case of a restart!\n");
+
+	if (srec_dlg.register_dlgcb(sess->dlg, DLGCB_PROCESS_VARS,
+			srec_dlg_read_callback, sess, NULL))
+		LM_WARN("cannot register callback for session de-serialization! "
+			"Will not be able to handle in-dialog for replicated sessions!\n");
+	LM_DBG("registered dialog callbacks for %p\n", sess);
 	sess->flags |= SIPREC_DLG_CBS;
 	return 0;
 }
@@ -445,9 +463,15 @@ static int srs_send_invite(struct src_sess *sess)
 	ci.method.len = INVITE_LEN;
 	/* try the first srs_uri */
 	ci.req_uri = SIPREC_SRS(sess);
-	/* TODO: fix uris */
-	ci.to_uri = ci.req_uri;
-	ci.from_uri = ci.to_uri;
+
+	if (sess->from_uri.len)
+		ci.from_uri = sess->from_uri;
+	else
+		ci.from_uri = ci.req_uri;
+	if (sess->to_uri.len)
+		ci.to_uri = sess->to_uri;
+	else
+		ci.to_uri = ci.req_uri;
 	if (sess->headers.s) {
 		hdrs.s = pkg_malloc(extra_headers.len + sess->headers.len);
 		if (!hdrs.s) {
@@ -718,10 +742,6 @@ int src_pause_recording(void)
 	/* mark the session as being paused */
 	sess->flags |= SIPREC_PAUSED;
 	ret = src_update_recording(NULL, sess);
-	if(ret < 0) {
-		LM_ERR("cannot pause recording! session_id[%s]\n", sess->uuid);
-		sess->flags &= ~SIPREC_PAUSED;
-	}
 
 end:
 	SIPREC_UNLOCK(sess);
@@ -746,10 +766,6 @@ int src_resume_recording(void)
 	}
 	sess->flags &= ~SIPREC_PAUSED;
 	ret = src_update_recording(NULL, sess);
-	if(ret < 0) {
-		LM_ERR("cannot resume recording! session_id[%s]\n", sess->uuid);
-		sess->flags |= SIPREC_PAUSED;
-	}
 
 end:
 	SIPREC_UNLOCK(sess);
