@@ -493,6 +493,9 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 			case HDR_MIN_EXPIRES_T:
 			case HDR_FEATURE_CAPS_T:
 			case HDR_REPLACES_T:
+			case HDR_SECURITY_CLIENT_T:
+			case HDR_SECURITY_SERVER_T:
+			case HDR_SECURITY_VERIFY_T:
 				/* we ignore them for now even if they have something parsed*/
 				break;
 
@@ -975,6 +978,24 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len,
 					new_msg->replaces = new_hdr;
 				}
 				break;
+			case HDR_SECURITY_CLIENT_T:
+				if (HOOK_NOT_SET(security_client))
+					new_msg->security_client = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_client, new_hdr);
+				break;
+			case HDR_SECURITY_SERVER_T:
+				if (HOOK_NOT_SET(security_server))
+					new_msg->security_server = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_server, new_hdr);
+				break;
+			case HDR_SECURITY_VERIFY_T:
+				if (HOOK_NOT_SET(security_verify))
+					new_msg->security_verify = new_hdr;
+				else
+					LINK_SIBLING_HEADER(security_verify, new_hdr);
+				break;
 			default:
 				/* ignore the rest*/
 				;
