@@ -49,6 +49,7 @@
 #include "../../globals.h"   /* is_main */
 #include "../../ut.h"        /* str_init */
 #include "../../ipc.h"
+#include "../../pvar.h"
 
 #include "ul_mod.h"
 #include "dlist.h"           /* register_udomain */
@@ -61,6 +62,7 @@
 #include "ul_mi.h"
 #include "ul_callback.h"
 #include "usrloc.h"
+#include "kv_store.h"
 
 #define CONTACTID_COL  "contact_id"
 #define USER_COL       "username"
@@ -91,6 +93,9 @@ int ul_check_config(void);
 int ul_check_db(void);
 int ul_deprec_shp(modparam_t _, void *modparam);
 
+/*! \brief Fixup functions */
+static int domain_fixup(void** param);
+
 //static int add_replication_dest(modparam_t type, void *val);
 
 extern int bind_usrloc(usrloc_api_t* api);
@@ -187,6 +192,26 @@ int latency_event_min_us;
  */
 static const cmd_export_t cmds[] = {
 	{"ul_bind_usrloc", (cmd_function)bind_usrloc, {{0,0,0}},0},
+        {"ul_add_key", (cmd_function)w_add_key, {
+                {CMD_PARAM_STR|CMD_PARAM_STATIC, domain_fixup, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {CMD_PARAM_STR|CMD_PARAM_OPT, 0, 0},
+                {0,0,0}},
+                ALL_ROUTES},
+        {"ul_get_key", (cmd_function)w_get_key, {
+                {CMD_PARAM_STR|CMD_PARAM_STATIC, domain_fixup, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {CMD_PARAM_VAR, 0, 0},
+                {0,0,0}},
+                ALL_ROUTES},
+       {"ul_del_key", (cmd_function)w_delete_key, {
+                {CMD_PARAM_STR|CMD_PARAM_STATIC, domain_fixup, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {CMD_PARAM_STR, 0, 0},
+                {0,0,0}},
+                ALL_ROUTES},
 	{0,0,{{0,0,0}},0}
 };
 
@@ -492,44 +517,29 @@ static int mi_child_init(void)
  */
 static void destroy(void)
 {
-	int do_sync = 0;
-
 	/* we need to sync DB in order to flush the cache */
 	if (have_sql_con() && ul_dbf.init) {
 		ul_dbh = ul_dbf.init(&db_url); /* Get a new database connection */
 		if (!ul_dbh) {
 			LM_ERR("failed to connect to database\n");
-			do_sync = -10;
-		} else {
-			do_sync++;
+	} else {
+			ul_unlock_locks();
+			if (sync_lock)
+				lock_start_read(sync_lock);
+			if (_synchronize_all_udomains() != 0) {
+				LM_ERR("flushing cache failed\n");
+			}
+			if (sync_lock) {
+				lock_stop_read(sync_lock);
+				lock_destroy_rw(sync_lock);
+				sync_lock = 0;
+			}
+			ul_dbf.close(ul_dbh);
 		}
 	}
 
-	if (have_cdb_con()) {
-		if (init_cachedb() < 0)
-			do_sync = -10;
-		else
-			do_sync++;
-	}
-
-	ul_unlock_locks();
-
-	if (sync_lock)
-		lock_start_read(sync_lock);
-	if (do_sync > 0 && _synchronize_all_udomains() != 0)
-		LM_ERR("flushing cache failed\n");
-	if (sync_lock) {
-		lock_stop_read(sync_lock);
-		lock_destroy_rw(sync_lock);
-		sync_lock = NULL;
-	}
-
-	if (ul_dbh)
-		ul_dbf.close(ul_dbh);
 	if (cdbc)
 		cdbf.destroy(cdbc);
-
-	ul_dbh = NULL;
 	cdbc = NULL;
 
 	free_all_udomains();
@@ -948,3 +958,25 @@ int ul_check_db(void)
 
 	return 0;
 }
+
+/*! \brief
+ * Convert char* parameter to udomain_t* pointer
+ */
+static int domain_fixup(void** param)
+{
+        udomain_t* d;
+        str d_nt;
+
+        if (pkg_nt_str_dup(&d_nt, (str*)*param) < 0)
+                return E_OUT_OF_MEM;
+
+        if (register_udomain(d_nt.s, &d) < 0) {
+                LM_ERR("failed to register domain\n");
+                return E_UNSPEC;
+        }
+
+        pkg_free(d_nt.s);
+
+        *param = (void*)d;
+        return 0;
+}
